pub struct Database(/* private fields */);Expand description
Implementations
Sourceimpl Database
impl Database
Sourcepub async fn connect(
configuration: DatabaseConfiguration,
) -> Result<Self, Error>
pub async fn connect( configuration: DatabaseConfiguration, ) -> Result<Self, Error>
Connects to the database using configuration
Sourcepub async fn start_transaction(&self) -> Result<Transaction, Error>
pub async fn start_transaction(&self) -> Result<Transaction, Error>
Starts a new transaction
&mut Transaction implements Executor like &Database does
but its database operations can be reverted using Transaction::rollback
or simply dropping the transaction without calling Transaction::commit.
Sourcepub async fn close(self)
pub async fn close(self)
Closes the database connection
While calling this method is not strictly necessary, terminating your program without it might result in some final queries not being flushed properly.
This method consumes the database handle,
but actually all handles created using clone will become invalid after this call.
This means any further operation would result in an Err