pub struct DatabaseConfiguration {
    pub driver: DatabaseDriver,
    pub min_connections: u32,
    pub max_connections: u32,
    pub disable_logging: Option<bool>,
    pub statement_log_level: Option<LevelFilter>,
    pub slow_statement_log_level: Option<LevelFilter>,
}
Configuration to create a database connection.

min_connections and max_connections must be greater than 0 and max_connections must be greater or equals min_connections.

Fields

driver: DatabaseDriver

The driver and its corresponding settings

min_connections: u32

Minimal connections to initialize upfront.

max_connections: u32

Maximum connections that allowed to be created.

disable_logging: Option<bool>

If set to true, logging will be completely disabled.

In case of None, false will be used.

statement_log_level: Option<LevelFilter>

Set the log level of SQL statements

In case of None, LevelFilter::Debug will be used.

slow_statement_log_level: Option<LevelFilter>

Log level in case of slow statements (>300 ms)

In case of None, LevelFilter::Warn will be used.

Implementations

Create a new database configuration with some defaults set.

Defaults:

Parameter: