pub type JoinTable<'until_build, 'post_build> = JoinTableData<'until_build, 'post_build>;Expand description
Type alias for JoinTableData.
As all databases use currently the same fields, a type alias is sufficient.
pub type JoinTable<'until_build, 'post_build> = JoinTableData<'until_build, 'post_build>;Type alias for JoinTableData.
As all databases use currently the same fields, a type alias is sufficient.