Module rorm_db::executor

source ·
This module defines a wrapper for sqlx’s Executor

Unlike sqlx’s Executor which provides several separate methods for different querying strategies, our Executor has a single method which is generic using the QueryStrategy trait.

Structs

QueryStrategy returning how many rows have been affected by the query
QueryStrategy returning a vector of rows
QueryStrategy returning nothing
QueryStrategy returning a single row
QueryStrategy returning an optional row
QueryStrategy returning a stream of rows

Traits

Some kind of database connection which can execute queries
Define how a query is send to and results retrieved from the database.