pub trait Executor<'executor> {
    fn execute<'data, 'result, Q>(
        self,
        query: String,
        values: Vec<Value<'data>>
    ) -> Q::Result<'result>
    where
        Q: QueryStrategy,
        'executor: 'result,
        'data: 'result
; }
Some kind of database connection which can execute queries

This trait is implemented by the database connection itself as well as transactions.

Required Methods

Execute a query

db.execute::<All>("SELECT * FROM foo;".to_string(), vec![]);

Implementors