Struct rorm_db::row::Row

source ·
pub struct Row(_);
Represents a single row from the database.

Implementations

Index into the database row and decode a single value.

A string index can be used to access a column by name and a usize index can be used to access a column by position.