pub struct Transaction { /* private fields */ }Expand description
Transactions can be used to provide a safe way to execute multiple SQL operations after another with a way to go back to the start without something changed in the database.
Can be obtained using Database::start_transaction.
Implementations
Sourceimpl Transaction
impl Transaction
Sourceimpl Transaction
impl Transaction
Sourcepub fn hooks(&mut self) -> SimpleHooksApi<'_>
pub fn hooks(&mut self) -> SimpleHooksApi<'_>
Accesses the simple API for adding hooks to the transaction
If you reach the API’s limits, consider Transaction::adv_hooks.
Sourcepub fn adv_hooks(&mut self) -> AdvancedHooksApi<'_>
pub fn adv_hooks(&mut self) -> AdvancedHooksApi<'_>
Accesses the advanced API for adding hooks to the transaction
If you’re new to transaction hooks, consider Transaction::hooks.